package com.zhongshu.reward.server.core.service;

import cn.hutool.core.lang.Snowflake;
import com.github.microservice.core.util.JsonUtil;
import com.wechat.pay.java.core.Config;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.exception.HttpException;
import com.wechat.pay.java.core.exception.MalformedMessageException;
import com.wechat.pay.java.core.exception.ServiceException;
import com.wechat.pay.java.core.util.IOUtil;
import com.wechat.pay.java.service.transferbatch.TransferBatchService;
import com.wechat.pay.java.service.transferbatch.model.*;
import com.zhongshu.reward.client.model.wallet.TransferModel;
import com.zhongshu.reward.client.model.wallet.TransferStatusModel;
import com.zhongshu.reward.server.core.config.WeChatConfig;
import com.zhongshu.reward.server.core.dao.WxTransferLogDao;
import com.zhongshu.reward.server.core.domain.WxTransferLog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UncheckedIOException;


@Service
@Slf4j
public class ChatTransferBatchService {

    @Autowired
    WeChatConfig weChatConfig;


//    @Autowired
//    TransferBatchService service;

    public static TransferBatchService service;

    @Autowired
    Snowflake snowflake;

    @Autowired
    WxTransferLogDao wxTransferLogDao;



    public  Config RSAAutoCertificateConfig(){
//        log.info(Thread.currentThread().getContextClassLoader().getResource("").getPath() + weChatConfig.getPrivateKey());


        // 初始化商户配置

        return new  RSAAutoCertificateConfig.Builder()
                        .merchantId(weChatConfig.getMerchantId())
                        // 使用 com.wechat.pay.java.core.util
                        // 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKey(privateKey(weChatConfig.getPrivateKey()))
//                        .privateKeyFromPath(Thread.currentThread().getContextClassLoader().getResource("").getPath() + weChatConfig.getPrivateKey())
                        .merchantSerialNumber(weChatConfig.getMerchantSerialNumber())
                        .apiV3Key(weChatConfig.getApiV3Key())
                        .build();

    }

    private String privateKey(String keyPath) {

        try {
            InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream(keyPath);
//            FileInputStream inputStream = new FileInputStream(keyPath);
            Throwable var2 = null;

            String var3;
            try {
                var3 = IOUtil.toString(inputStream);
            } catch (Throwable var13) {
                var2 = var13;
                throw var13;
            } finally {
                if (inputStream != null) {
                    if (var2 != null) {
                        try {
                            inputStream.close();
                        } catch (Throwable var12) {
                            var2.addSuppressed(var12);
                        }
                    } else {
                        inputStream.close();
                    }
                }

            }

            return var3;
        } catch (IOException var15) {
            throw new UncheckedIOException(var15);
        }
    }




    /**
     * 通过微信批次单号查询批次单
     */
    public  TransferBatchEntity getTransferBatchByNo(TransferStatusModel transferStatusModel) {
        Config config = RSAAutoCertificateConfig();
        // 初始化服务
        service = new TransferBatchService.Builder().config(config).build();

        GetTransferBatchByNoRequest request = new GetTransferBatchByNoRequest();
        BeanUtils.copyProperties(transferStatusModel, request);
        return service.getTransferBatchByNo(request);
    }

//    /**
//     * 通过商家批次单号查询批次单
//     */
//    public  TransferBatchEntity getTransferBatchByOutNo() {
//
//        GetTransferBatchByOutNoRequest request = new GetTransferBatchByOutNoRequest();
//        return service.getTransferBatchByOutNo(request);
//    }

    /**
     * 发起商家转账
     */
    public InitiateBatchTransferResponse initiateBatchTransfer(TransferModel transferModel, String appid) {
        WxTransferLog wxTransferErrorLog = new WxTransferLog();
        Config config = RSAAutoCertificateConfig();
        // 初始化服务
        service = new TransferBatchService.Builder().config(config).build();
        try {
            InitiateBatchTransferRequest request = new InitiateBatchTransferRequest();
            BeanUtils.copyProperties(transferModel,request);
            request.setAppid(appid);
            request.setTransferSceneId(transferModel.getTransferSceneId());
            request.setTransferSceneId(weChatConfig.getTransferSceneId());
            InitiateBatchTransferResponse initiateBatchTransferResponse = service.initiateBatchTransfer(request);

            wxTransferErrorLog.setRequestBody(JsonUtil.toJson(request));
            wxTransferErrorLog.setResultContent(JsonUtil.toJson(initiateBatchTransferResponse));
            wxTransferErrorLog.setResultState("SUCCESS");
            wxTransferLogDao.save(wxTransferErrorLog);
            return initiateBatchTransferResponse;
        } catch (HttpException e) { // 发送HTTP请求失败
            // 调用e.getHttpRequest()获取请求打印日志或上报监控，更多方法见HttpException定义
            log.info("发送HTTP请求失败：{}", e.getHttpRequest());
            wxTransferErrorLog.setRequestBody(JsonUtil.toJson(transferModel));
            wxTransferErrorLog.setResultContent(JsonUtil.toJson(e.getHttpRequest()));
        } catch (ServiceException e) { // 服务返回状态小于200或大于等于300，例如500
            // 调用e.getResponseBody()获取返回体打印日志或上报监控，更多方法见ServiceException定义
            log.info("请求失败2：{}", e.getResponseBody());
            wxTransferErrorLog.setRequestBody(JsonUtil.toJson(transferModel));
            wxTransferErrorLog.setResultContent(e.getResponseBody());
        } catch (MalformedMessageException e) { // 服务返回成功，返回体类型不合法，或者解析返回体失败
            // 调用e.getMessage()获取信息打印日志或上报监控，更多方法见MalformedMessageException定义
            log.info("请求失败3：{}", e.getMessage());
            wxTransferErrorLog.setRequestBody(JsonUtil.toJson(transferModel));
            wxTransferErrorLog.setResultContent(e.getMessage());
        }
        wxTransferErrorLog.setResultState("Exception");
        wxTransferLogDao.save(wxTransferErrorLog);
        return null;
    }

//    /**
//     * 通过微信明细单号查询明细单
//     */
//    public  TransferDetailEntity getTransferDetailByNo() {
//
//        GetTransferDetailByNoRequest request = new GetTransferDetailByNoRequest();
//        return service.getTransferDetailByNo(request);
//    }
//
//    /**
//     * 通过商家明细单号查询明细单
//     */
//    public  TransferDetailEntity getTransferDetailByOutNo() {
//
//        GetTransferDetailByOutNoRequest request = new GetTransferDetailByOutNoRequest();
//        return service.getTransferDetailByOutNo(request);
//    }
}