package com.zhongshu.reward.server.core.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.Snowflake;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailCompact;
import com.zhongshu.reward.client.model.TransferRulerModel;
import com.zhongshu.reward.client.ret.ResultContent;
import com.github.microservice.auth.security.helper.AuthHelper;
import com.github.microservice.components.data.mongo.mongo.helper.DBHelper;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import com.wechat.pay.java.service.transferbatch.model.TransferBatchEntity;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailInput;
import com.zhongshu.reward.client.model.param.WalletTransferParam;
import com.zhongshu.reward.client.model.wallet.TransferModel;
import com.zhongshu.reward.client.model.wallet.TransferStatusModel;
import com.zhongshu.reward.client.model.wallet.WxTransferBatchModel;
import com.zhongshu.reward.client.ret.CommentException;
import com.zhongshu.reward.client.ret.ResultState;
import com.zhongshu.reward.client.type.OperateType;
import com.zhongshu.reward.client.type.TimeUnitType;
import com.zhongshu.reward.client.type.TransferChannel;
import com.zhongshu.reward.client.type.TransferStatus;
import com.zhongshu.reward.server.core.config.WeChatConfig;
import com.zhongshu.reward.server.core.dao.OperateDisableDao;
import com.zhongshu.reward.server.core.dao.TransferRulerDao;
import com.zhongshu.reward.server.core.dao.WalletDao;
import com.zhongshu.reward.server.core.dao.WxTransferBatchDao;
import com.zhongshu.reward.server.core.domain.OperateDisable;
import com.zhongshu.reward.server.core.domain.TransferRuler;
import com.zhongshu.reward.server.core.domain.Wallet;
import com.zhongshu.reward.server.core.domain.WxTransferBatch;
import com.zhongshu.reward.server.core.helper.RedisHelper;
import com.zhongshu.reward.server.core.util.DateUtils;
import com.zhongshu.reward.server.core.util.JedisUtil;
import com.zhongshu.reward.server.core.util.wx.BusinessUtil;
import io.swagger.models.Model;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wjf
 * @date 2024/8/5
 */
@Service
@Slf4j
public class WxTransferService {

    @Autowired
    WalletDao walletDao;

    @Autowired
    ChatTransferBatchService wechatService;

    @Autowired
    Snowflake snowflake;

    @Autowired
    WxTransferBatchDao wxTransferBatchDao;

    @Autowired
    TransferRulerDao transferRulerDao;

    @Autowired
    WeChatConfig weChatConfig;

    @Autowired
    OperateDisableDao operateDisableDao;

    @Autowired
    AuthHelper authHelper;

    @Autowired
    DBHelper dbHelper;

    @Autowired
    RedisHelper redisHelper;

    @Autowired
    JedisUtil jedisUtil;

    @Autowired
    WalletService walletService;



    public ResultContent transferLock(WalletTransferParam param){
        String userId = authHelper.getCurrentUser().getUserId();
        String key = BusinessUtil.WXTRANSFER + ":" + userId;
        String requestId = "";
        try {
            requestId = snowflake.nextIdStr();
            boolean ret = false;
            ret = jedisUtil.distributedLock(key, requestId, 30);
            if (ret){
                return transfer(param);
            }else {
                return ResultContent.buildFail("操作频繁，请稍后再试");
            }
        }catch (Exception e){
            e.printStackTrace();
            return ResultContent.buildFail("系统异常，请联系管理员");
        } finally {
            jedisUtil.assignDb(BusinessUtil.REDIS_DB).releaseDistributedLock(key, requestId);
        }
    }

    /**
     * 发起提现
     * @param param
     * @return
     */
    public ResultContent transfer(WalletTransferParam param){
        String userId = authHelper.getCurrentUser().getUserId();
        //校验提现操作是否被禁用
        OperateDisable operateDisable = checkOperateDisable(userId, OperateType.TRANSFER);
        if (null != operateDisable){
            return ResultContent.build(ResultState.Fail, "系统异常，请"+ DateUtils.paresTime(operateDisable.getTTL().getTime(), DateUtils.patternyyyyMis) + "后重试");
        }

        Wallet wallet = walletDao.findByUserId(userId);
        if (ObjectUtils.isEmpty(wallet)){
            return ResultContent.buildFail("钱包暂未开通");
        }
        //校验提现规则
        ResultContent validResult = validTransfer(param.getTotal(), wallet);
        if (validResult.isFailed()){
            return validResult;
        }

        String outBatchNo = snowflake.nextIdStr();
        String outDetailNo = snowflake.nextIdStr();

        TransferModel transferModel = new TransferModel();
        transferModel.setBatchName("佣金/返利发放");
        transferModel.setBatchRemark("佣金/返利发放");
        transferModel.setTotalAmount(param.getTotal().longValue());
        transferModel.setOutBatchNo(outBatchNo);

        List<TransferDetailInput> detailList = getTransferDetailInputs(param, outDetailNo);
        transferModel.setTotalNum(detailList.size());
        transferModel.setTransferDetailList(detailList);

        //初始化提现单
        WxTransferBatch wxTransferBatch = getWxTransferBatch(param, wallet, outBatchNo, outDetailNo);

        InitiateBatchTransferResponse firstResponse = wechatService.initiateBatchTransfer(transferModel, param.getAppid());
        if (firstResponse==null){
            //失败做一次补偿
            firstResponse = wechatService.initiateBatchTransfer(transferModel, param.getAppid());
            if (firstResponse == null){
                //禁用1小时
                disableTransfer(userId);
                wxTransferBatch.setTransferStatus(TransferStatus.FAIL);
                wxTransferBatch.setFailReason("提现失败请一小时后再次尝试");
                wxTransferBatchDao.save(wxTransferBatch);
                return ResultContent.buildFail("提现失败请一小时后再次尝试");
            }
        }

        wxTransferBatch.setBatchStatus(firstResponse.getBatchStatus());
        wxTransferBatch.setWxBatchId(firstResponse.getBatchId());
        wxTransferBatch.setTransferStatus(TransferStatus.PROCESSING);
        wxTransferBatchDao.save(wxTransferBatch);

//        Wallet currentWallet = walletDao.findTop1ById(param.getWalletId());
//        wallet.setAmount(currentWallet.getAmount().subtract(param.getTotal()));
//        wallet.setTodayTransferAmount(currentWallet.getTodayTransferAmount().add(param.getTotal()));
//        walletDao.save(wallet);
        BigDecimal negateTotal = param.getTotal().negate();
        walletDao.updateIncAmount(wallet.getId(), negateTotal);
        return ResultContent.build(ResultState.Success);
    }

    private void disableTransfer(String userId) {
        OperateDisable newDisable = new OperateDisable();
        newDisable.setUserId(userId);
        newDisable.setType(OperateType.TRANSFER);
        newDisable.setTTL(new Date(this.dbHelper.getTime() + 60*60 * 1000L));
        operateDisableDao.save(newDisable);
    }

    @NotNull
    private static WxTransferBatch getWxTransferBatch(WalletTransferParam param, Wallet wallet, String outBatchNo, String outDetailNo) {
        WxTransferBatch wxTransferBatch = new WxTransferBatch();
        wxTransferBatch.setBatchNo(outBatchNo);
        wxTransferBatch.setDetailNo(outDetailNo);
        wxTransferBatch.setWallet(wallet);
        wxTransferBatch.setTotal(param.getTotal());
        wxTransferBatch.setChannel(TransferChannel.WeChat);
        wxTransferBatch.setBeforeWalletInfo(wallet);
        wxTransferBatch.setWechatName(param.getWechatName());
        return wxTransferBatch;
    }

    @NotNull
    private static List<TransferDetailInput> getTransferDetailInputs(WalletTransferParam param, String outDetailNo) {
        List<TransferDetailInput> detailList = new ArrayList<>();
        TransferDetailInput detail = new TransferDetailInput();
        detail.setTransferAmount(param.getTotal().longValue());
        detail.setTransferRemark("佣金/返利发放");
        detail.setOpenid(param.getOpenid());
        detail.setOutDetailNo(outDetailNo);
        detailList.add(detail);
        return detailList;
    }

    public OperateDisable checkOperateDisable(String userId, OperateType operateType){
        OperateDisable operateDisable = operateDisableDao.findTopByUserIdAndType(userId, operateType);
        if (ObjectUtils.isEmpty(operateDisable)){
            return null;
        }else if (new Date().getTime() < operateDisable.getTTL().getTime()){
            return operateDisable;
        }else {
            operateDisableDao.delete(operateDisable);
            return null;
        }
    }


    public Object queryTransfer(String walletId, Integer year, Integer month){
        Long monthStartTime = DateUtils.getMonthStartTime(year, month);
        Long monthEndTime = DateUtils.getMonthEndTime(year, month);
        List<WxTransferBatch> list = wxTransferBatchDao.list(walletId, monthStartTime, monthEndTime);
        return ResultContent.buildContent(list.stream().map(this::toModel).collect(Collectors.toList()));
    }


    public void updateTransferStatus(String transferId){
        WxTransferBatch wxTransferBatch = wxTransferBatchDao.findTop1ById(transferId);
        if (ObjectUtils.isEmpty(wxTransferBatch)){
            return;
        }

        TransferStatus transferStatus = wxTransferBatch.getTransferStatus();
        if (!transferStatus.equals(TransferStatus.PROCESSING)){
           return;
        }

        TransferStatusModel transferStatusModel = new TransferStatusModel();
        transferStatusModel.setBatchId(wxTransferBatch.getWxBatchId());
        transferStatusModel.setNeedQueryDetail(true);
        transferStatusModel.setDetailStatus("ALL");
        TransferBatchEntity transferBatchByNo = wechatService.getTransferBatchByNo(transferStatusModel);

        String batchStatus = transferBatchByNo.getTransferBatch().getBatchStatus();
        String detailStatus = "INIT";
        List<TransferDetailCompact> transferDetailList = transferBatchByNo.getTransferDetailList();
        if (null != transferDetailList && !transferDetailList.isEmpty()){
            detailStatus = transferBatchByNo.getTransferDetailList().get(0).getDetailStatus();
        }

        String walletId = wxTransferBatch.getWallet().getId();
        BigDecimal total = wxTransferBatch.getTotal();

        if(batchStatus.equals("FINISHED") && detailStatus.equals("SUCCESS")){
            transferStatus = TransferStatus.SUCCESS;
        }else if(batchStatus.equals("CLOSED")){
            transferStatus = TransferStatus.CLOSE;
            walletDao.updateIncAmount(walletId, total);
        }else if (detailStatus.equals("FAIL")){
            transferStatus = TransferStatus.FAIL;
            walletDao.updateIncAmount(walletId, total);
        }
        boolean update = wxTransferBatchDao.updateStatus(transferId, batchStatus, detailStatus, transferStatus);
        log.info("更新微信转账批次状态：结果：{}， id:{}, batchStatus：{}, detailStatus: {}, transferStatus: {}",update, transferId, batchStatus, detailStatus, transferStatus);
    }

    WxTransferBatchModel toModel(WxTransferBatch wxTransferBatch){
        WxTransferBatchModel model = new WxTransferBatchModel();
        if (null != wxTransferBatch){
            BeanUtil.copyProperties(wxTransferBatch, model, "beforeWalletInfo", "wallet");
            model.setWalletId(wxTransferBatch.getWallet().getId());
        }
        return model;
    }


    public Object transferRulerInfo(){
        List<TransferRuler> list = transferRulerDao.findAll();
        if (list.isEmpty()){
            return ResultContent.buildFail("管理员未设置提现规则");
        }
        TransferRuler transferRuler = list.get(0);
        TransferRulerModel model = new TransferRulerModel();
        if (ObjectUtils.isNotEmpty(transferRuler)){
            BeanUtil.copyProperties(transferRuler, model);
        }
        String userId = authHelper.getCurrentUser().getUserId();
        Wallet wallet = walletService.getWalletByUserId(userId);
        ResultContent resultContent = validTransfer(new BigDecimal("30"), wallet);
        model.setIsTransfer(resultContent.getState().equals(ResultState.Success));
        model.setRemark(resultContent.getMsg());
        model.setTotal(wallet.getAmount());
        return ResultContent.buildContent(model);
    }

    public ResultContent validTransfer(BigDecimal total, Wallet wallet){
        long currentTime = System.currentTimeMillis();
        Integer size = 0;
        List<TransferRuler> list = transferRulerDao.findAll();
        if (list.isEmpty()){
            return ResultContent.buildFail("管理员未设置提现规则");
        }
        TransferRuler transferRuler = list.get(0);

        //判断可提现金额是否足够
        if (total.compareTo(wallet.getAmount()) > 0){
            return ResultContent.buildFail("可提现余额不足");
        }
        //单笔金额
        if (total.compareTo(transferRuler.getMinTotal()) < 0){
            return ResultContent.buildFail("单笔提现金额低于最小值");
        }else if (total.compareTo(transferRuler.getMaxTotal()) > 0){
            return ResultContent.buildFail("单笔提现金额超出最大值");
        }
        //提现时间
        if (transferRuler.getTimeUnit().equals(TimeUnitType.MONTH)){//每月
            Long dayOfMonthStartTime = DateUtils.getDayOfMonthStartTime(transferRuler.getStartDay());
            Long dayOfMonthEndTime = DateUtils.getDayOfMonthEndTime(transferRuler.getEndDay());
            if (currentTime < dayOfMonthStartTime || currentTime > dayOfMonthEndTime){
                return ResultContent.buildFail("当前不处于提现时间段");
            }
            size = wxTransferBatchDao.countByTime(wallet.getId(), dayOfMonthStartTime, dayOfMonthEndTime);
        }else if (transferRuler.getTimeUnit().equals(TimeUnitType.DAY)){
            Long hourMinuteStartTime = DateUtils.getHourMinuteStartTime(transferRuler.getStartHour(), transferRuler.getStartMinute());
            Long hourMinuteEndTime = DateUtils.getHourMinuteEndTime(transferRuler.getEndHour(), transferRuler.getEndMinute());
            if (currentTime < hourMinuteStartTime || currentTime > hourMinuteEndTime){
                return ResultContent.buildFail("当前不处于提现时间段");
            }
            size = wxTransferBatchDao.countByTime(wallet.getId(), hourMinuteStartTime, hourMinuteEndTime);
        }
        //判断提现额度是否超出每日上限
        Integer sumDayTotal = wxTransferBatchDao.sumDayTotal(wallet.getId(), DateUtils.getCurrentDayStartTime().getTime(), DateUtils.getCurrentDayEndTime().getTime());
        if (total.add(BigDecimal.valueOf(sumDayTotal)).compareTo(transferRuler.getDayMaxTotal()) > 0){
            return ResultContent.buildFail("超出每日提现金额上限");
        }
        //提现次数
        if (size >= transferRuler.getSize()){
            return ResultContent.buildFail("超过可提现次数");
        }
        return ResultContent.buildSuccess();
    }


}
