package com.zhongshu.reward.server.core.util;

import cn.hutool.log.Log;
import lombok.extern.slf4j.Slf4j;
import org.bson.types.ObjectId;

import java.security.SecureRandom;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Base64;
import java.util.Calendar;
import java.util.Date;

/**
 * @author wjf
 * @date 2024/8/8
 */
@Slf4j
public class DateUtils {

    public final static String FORMAT_LONG = "yyyy-MM-dd HH:mm:ss";

    public final static String patternyyyyMis = "yyyy-MM-dd HH:mm";
    public static Long timeToLong(String time, String format) {
        if (!org.springframework.util.StringUtils.hasText(time)) {
            return null;
        }
        try {
            return new SimpleDateFormat(format).parse(time).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 获取本月开始时间
     */
    public static Long getCurrentMonthStartTime(){
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_MONTH, 1); // 设置为本月的第一天
        calendar.set(Calendar.HOUR_OF_DAY, 0); // 将小时设置为0
        calendar.set(Calendar.MINUTE, 0); // 将分钟设置为0
        calendar.set(Calendar.SECOND, 0); // 将秒设置为0
        calendar.set(Calendar.MILLISECOND, 0); // 将毫秒设置为0
        return calendar.getTimeInMillis();
    }

    /**
     * 获取上月指定日期时间
     */
    public static Long last(Long time){
        Date date = new Date(time);
        String year = String.format("%tY", date);
        String day = String.format("%td", date);
        String month = String.format("%tm", date);
        String hour = String.format("%tH", date);
        String m = String.format("%tM", date);
        String s = String.format("%tS", date);
        int yearInt = Integer.parseInt(year);
        int monthInt = Integer.parseInt(month);
        int dayInt = Integer.parseInt(day);
        int hourInt = Integer.parseInt(hour);
        int mInt = Integer.parseInt(m);
        int sInt = Integer.parseInt(s);


        Calendar calendar = Calendar.getInstance();
        calendar.set(yearInt, monthInt-2, dayInt, hourInt, mInt, sInt);
//        calendar.set(Calendar.MONTH, Integer.getInteger(month)-1);
        return calendar.getTimeInMillis();
    }


    /**
     * 判断当日是否当月最后一天
     */
    public static boolean checkLastDayOfMonth(){
        Calendar calendar = Calendar.getInstance();
        int today = calendar.get(Calendar.DATE);
        calendar.set(Calendar.DATE, 1);
        calendar.add(Calendar.MONTH, 1);
        calendar.set(Calendar.DATE, 1);
        int lastDay = calendar.get(Calendar.DATE);
        return today == lastDay;
    }


    /**
     * 获得本天的开始时间
     *
     * @return
     */
    public static Date getCurrentDayStartTime() {
        ZoneId zoneId = ZoneId.systemDefault();
        LocalDateTime date = LocalDateTime.now();
        LocalDateTime startOfTheDay = LocalDateTime.of(date.toLocalDate(), LocalTime.MIN);
        Date beginTime = Date.from(startOfTheDay.atZone(zoneId).toInstant());
        return beginTime;
    }

    /**
     * 获得本天的结束时间
     *
     * @return
     */
    public static Date getCurrentDayEndTime() {
        ZoneId zoneId = ZoneId.systemDefault();
        LocalDateTime date = LocalDateTime.now();
        LocalDateTime startOfTheDay = LocalDateTime.of(date.toLocalDate(), LocalTime.MAX);
        Date beginTime = Date.from(startOfTheDay.atZone(zoneId).toInstant());
        return beginTime;
    }

    /**
     * 次月指定日开始时间
     *
     * @return
     */
    public static Long nextMonthDayStartTime(Integer day) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, 1);

        if (day >c.getActualMaximum(Calendar.DAY_OF_MONTH)){
            c.set(Calendar.DATE, c.getActualMaximum(Calendar.DAY_OF_MONTH));
        }else {
            c.set(Calendar.DATE, day);
        }
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    /**
     * 次月指定日开始时间
     *
     * @return
     */
    public static Long lastMonthDayStartTime(Integer day) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);

        if (day >c.getActualMaximum(Calendar.DAY_OF_MONTH)){
            c.set(Calendar.DATE, c.getActualMaximum(Calendar.DAY_OF_MONTH));
        }else {
            c.set(Calendar.DATE, day);
        }
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    /**
     * 上月开始时间
     *
     * @return
     */
    public static Long lastMonthStartTime() {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);
        c.set(Calendar.DATE, 1);

        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    /**
     * 上月结束时间
     *
     * @return
     */
    public static Long lastMonthEndTime() {
        Calendar c = Calendar.getInstance();

        c.add(Calendar.MONTH, 0);
        c.set(Calendar.DATE, 1);
        c.set(Calendar.DAY_OF_MONTH, 0);

        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);

        return c.getTimeInMillis();
    }

    /**
     * 本月指定日的开始时间
     *
     * @return
     */
    public static Long getDayOfMonthEndTime(Integer day) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.DATE, 1);
        c.add(Calendar.DATE, day-1);

        if (day >c.getActualMaximum(Calendar.DAY_OF_MONTH)){
            c.set(Calendar.DATE, c.getActualMaximum(Calendar.DAY_OF_MONTH));
        }else {
            c.set(Calendar.DATE, day);
        }

        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);

        return c.getTimeInMillis();
    }

    /**
     * 本月指定日的结束时间
     *
     * @return
     */
    public static Long getDayOfMonthStartTime(Integer day) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.DATE, 1);
        c.add(Calendar.DATE, day-1);

        if (day >c.getActualMaximum(Calendar.DAY_OF_MONTH)){
            c.set(Calendar.DATE, c.getActualMaximum(Calendar.DAY_OF_MONTH));
        }else {
            c.set(Calendar.DATE, day);
        }

        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);

        return c.getTimeInMillis();
    }

    /**
     * 指定年月开始时间
     *
     * @return
     */
    public static Long getMonthStartTime(Integer year, Integer month) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month - 1);
        c.set(Calendar.DATE, 1);

        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);

        return c.getTimeInMillis();

    }

    /**
     * 指定年月结束时间
     *
     * @return
     */
    public static Long getMonthEndTime(Integer year, Integer month) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month - 1);
        c.add(Calendar.MONTH, 1);
        c.set(Calendar.DAY_OF_MONTH, 0);

        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);
        return c.getTimeInMillis();
    }

    /**
     * 当天指定时分开始时间 time: HH:mm
     *
     * @return
     */
    public static Long getHourMinuteStartTime(Integer hour, Integer minute) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    /**
     * 当天指定时分结束时间 time: HH:mm
     *
     * @return
     */
    public static Long getHourMinuteEndTime(Integer hour, Integer minute) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    public static String paresTime(Long time, String pattern) {
        if (time == null || time <= 0) {
            return "";
        }
        Date date = new Date(time);
        SimpleDateFormat formatter = new SimpleDateFormat(pattern);
        return formatter.format(date);
    }

    public static void main(String[] args) {
        Long hourMinuteStartTime = getHourMinuteStartTime(23, 0);

        log.info(new ObjectId().toHexString());

        log.info(paresTime(last(1723898160215L), FORMAT_LONG));

        log.info(paresTime(getDayOfMonthStartTime(28), FORMAT_LONG));
        log.info(paresTime( getDayOfMonthEndTime(30), FORMAT_LONG));
//        log.info(paresTime(getDayOfMonthStartTime(2), FORMAT_LONG));
//        log.info(paresTime(getDayOfMonthEndTime(2), FORMAT_LONG));
    }
}
